package com.ether;

import com.medias.megapoli.utils.MegapoliInitialisation;
import gov.noaa.pmel.sgt.CartesianGraph;
import gov.noaa.pmel.sgt.ColorMap;
import gov.noaa.pmel.sgt.ContourLevels;
import gov.noaa.pmel.sgt.Graph;
import gov.noaa.pmel.sgt.GridAttribute;
import gov.noaa.pmel.sgt.IndexedColorMap;
import gov.noaa.pmel.sgt.JPane;
import gov.noaa.pmel.sgt.Layer;
import gov.noaa.pmel.sgt.LineAttribute;
import gov.noaa.pmel.sgt.LineCartesianRenderer;
import gov.noaa.pmel.sgt.LineKey;
import gov.noaa.pmel.sgt.LinearTransform;
import gov.noaa.pmel.sgt.PlainAxis;
import gov.noaa.pmel.sgt.SGLabel;
import gov.noaa.pmel.sgt.StackedLayout;
import gov.noaa.pmel.sgt.TimeAxis;
import gov.noaa.pmel.sgt.demo.TestData;
import gov.noaa.pmel.sgt.dm.SGTData;
import gov.noaa.pmel.sgt.dm.SGTMetaData;
import gov.noaa.pmel.sgt.dm.SimpleGrid;
import gov.noaa.pmel.sgt.dm.SimpleLine;
import gov.noaa.pmel.sgt.swing.JPlotLayout;
import gov.noaa.pmel.util.Dimension2D;
import gov.noaa.pmel.util.GeoDate;
import gov.noaa.pmel.util.GeoDateArray;
import gov.noaa.pmel.util.IllegalTimeValue;
import gov.noaa.pmel.util.Point2D;
import gov.noaa.pmel.util.Range2D;
import gov.noaa.pmel.util.SoTPoint;
import gov.noaa.pmel.util.SoTRange;
import gov.noaa.pmel.util.TimeRange;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * @author vmipsl
 * @date 05 july 2011
 */
public class EtherPlotServiceImpl
        implements EtherPlotService
{
    public static Integer getMaxWidth()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getWidth() ).intValue(), MAX_WIDTH );
    }

    public static Integer getMaxHeight()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getHeight() ).intValue(), MAX_HEIGHT );
    }

    /**
     * Create the main JPane with the 3 jPanes
     *
     * @param megapoliPlot
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createMainPane( @NotNull final MegapoliPlot megapoliPlot, @Nullable final Locale locale )
    {
        final Integer maxWidth = getMaxWidth();
        final Integer maxHeight = getMaxHeight();

        final JPane jPane = new JPane( "Main jPane", new Dimension( maxWidth, maxHeight ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );

        // Top Pane
        final JPane jPaneTop = createTopPane( megapoliPlot );
        jPane.add( jPaneTop, BorderLayout.NORTH );

        final Integer valuesNumber = megapoliPlot.getValuesNumber();

        // Bottom Pane
        final JPane jPaneBottom = createBottomPane( valuesNumber, locale );
        jPane.add( jPaneBottom, BorderLayout.SOUTH );

        // Center Pane
        final Integer centerHeight = maxHeight - jPaneTop.getHeight() - jPaneBottom.getHeight();
        if( valuesNumber > 0 )
        {
            final BufferedImage jPaneCenterBufferedImage = createCenterPane( megapoliPlot, maxWidth, centerHeight, locale );

            final ImageIcon imageIcon = new ImageIcon( jPaneCenterBufferedImage );
            final JLabel jLabelPlot = new JLabel( imageIcon, JLabel.CENTER );
            jLabelPlot.setBorder( BorderFactory.createLineBorder( Color.black, 1 ) );

            final JPane jPaneCenter = new JPane( "Center jPane", new Dimension( jPaneCenterBufferedImage.getWidth(), jPaneCenterBufferedImage.getHeight() ) );
            jPaneCenter.setLayout( new GridBagLayout() );
            jPaneCenter.add( jLabelPlot,
                    new GridBagConstraints( 0, 0, 1, 1, 0, 0,
                            GridBagConstraints.CENTER,
                            GridBagConstraints.CENTER,
                            new Insets( 0, 0, 0, 0 ), 0, 0 ) );
            jPane.add( jPaneCenter, BorderLayout.CENTER );
        }
        else
        {
            final JPane jPaneCenter = createEmptyCenterPane( locale );
            jPane.add( jPaneCenter, BorderLayout.CENTER );
        }

        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    /**
     * Create the top JPane with the logos and title
     *
     * @param megapoliPlot
     * @return
     */
    @NotNull
    public JPane createTopPane( @Nullable final MegapoliPlot megapoliPlot )
    {
        // Logos
        final ImageIcon logoMegapoli = new ImageIcon( megapoliPlot.getLogoMegapoli() );
        final JLabel jLabelLogoMegapoli = new JLabel( logoMegapoli );

        final ImageIcon logoEther = new ImageIcon( megapoliPlot.getLogoEther() );
        final JLabel jLabelLogoEther = new JLabel( logoEther );

        // Max height
        final Integer maxHeights = Math.max( logoMegapoli.getIconHeight(), logoEther.getIconHeight() );

        // jPane
        final JPane jPaneTop = new JPane( "Top jPane", new Dimension( 0, maxHeights ) );
        jPaneTop.setLayout( new BorderLayout() );

        jPaneTop.add( jLabelLogoMegapoli, BorderLayout.LINE_START );
        jPaneTop.add( jLabelLogoEther, BorderLayout.LINE_END );


        // Title
        final String formatedTitle = formatTitle( megapoliPlot.getTitle() );
        final JLabel jLabelTitle = new JLabel( formatedTitle, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, TITLE_FONT_SIZE );
        jLabelTitle.setFont( police );
        jPaneTop.add( jLabelTitle, BorderLayout.CENTER );

        return jPaneTop;
    }

    /**
     * Create the bottom JPane with the text "published .. (date)"
     *
     * @param dataNumber : the number of datas extracted from the base
     * @param locale
     * @return
     */
    @NotNull
    public JPane createBottomPane( @Nullable final Integer dataNumber, @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );

        final JPane jPaneBottom = new JPane();
        jPaneBottom.setLayout( new BorderLayout() );

        // Published
        final Calendar calendar = Calendar.getInstance();
        final String messagePublished = bundle.getString( "plot.published" );
        final JLabel jLabel = new JLabel( messagePublished + ' ' + DateHelper.formatDate( calendar.getTime(), DateHelper.ENGLISH_DATE_PATTERN ) + " " );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabel.setFont( police );
        jPaneBottom.add( jLabel, BorderLayout.EAST );

        if( null != dataNumber )
        {
            final String messageDataNumber = bundle.getString( "plot.dataNumber" );
            final JLabel jLabelDataNumber = new JLabel( messageDataNumber + ' ' + dataNumber );
            jLabelDataNumber.setFont( police );
            jPaneBottom.add( jLabelDataNumber, BorderLayout.WEST );
        }

        return jPaneBottom;
    }

    /**
     * Create an error JPane when plot can be created
     *
     * @param mainError
     * @param errorText
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createErrorPane( @NotNull final String mainError, @Nullable final String errorText, @Nullable final Locale locale )
    {
        // Image
        final ImageIcon errorImage = new ImageIcon( MegapoliInitialisation.pathImages + "/erreur.gif" );
        final JLabel jLabelErrorImage = new JLabel( errorImage );

        // Text
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String mainMessage = bundle.getString( "plot.errorMessage" );

        final JLabel jLabelError;
        if( null != errorText )
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "<br>" + errorText + "</html>", JLabel.CENTER );
        else
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "</html>", JLabel.CENTER );

        final Font police = new Font( "Arial", Font.BOLD, ERROR_FONT_SIZE );
        jLabelError.setFont( police );

        // jPane
        final JPane jPane = new JPane( "jPane", new Dimension( MAX_WIDTH, errorImage.getIconHeight() ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );
        jPane.add( jLabelErrorImage, BorderLayout.WEST );
        jPane.add( jLabelError, BorderLayout.CENTER );

        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    /**
     * Create the center JPane with no graph (if no value)
     *
     * @param locale
     * @return
     */
    @NotNull
    public JPane createEmptyCenterPane( @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String messageData = bundle.getString( "plot.noData" );

        final JPane jPaneCenter = new JPane();
        jPaneCenter.setLayout( new BorderLayout() );

        final JLabel jLabelData = new JLabel( messageData, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabelData.setFont( police );

        jPaneCenter.add( jLabelData, BorderLayout.CENTER );
        return jPaneCenter;
    }

    /**
     * Create the center JPane with the graph
     *
     * @param megapoliPlot
     * @param maxWidth
     * @param maxHeight
     * @return
     */
    @NotNull
    public BufferedImage createCenterPane( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer maxWidth, @NotNull final Integer maxHeight, @Nullable final Locale locale )
    {
//        final Integer plotWidth = Math.min( maxWidth - MARGIN_LEFT_RIGHT, maxWidth );
//        final Integer plotHeight = Math.min( maxHeight - MARGIN_LEFT_RIGHT, maxHeight );

//        final JPlotLayout jPlotLayout;
//        final BufferedImage bufferedImage;
//        if( megapoliPlot.isTimeSerie() )
//        {
//            jPlotLayout = createJPlotLayoutForTimeSerie( megapoliPlot, plotWidth, plotHeight );
//
//            // TODO : remove this change to bufferedImage to insert directly the jPlotLayout !!!!
//            bufferedImage = new BufferedImage( jPlotLayout.getWidth(), jPlotLayout.getHeight(), BufferedImage.TYPE_INT_RGB );
//            final Graphics2D graphics2D = bufferedImage.createGraphics();
//            jPlotLayout.draw( graphics2D );
//        }
//        else
//        {
//            jPlotLayout = createJPlotLayoutFor2D( megapoliPlot, plotWidth, plotHeight, locale );
//
//            final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPlotLayout );
//            bufferedImage = jPanelToImageManager.saveAsImage();
//        }
//
//        final ImageIcon imageIcon = new ImageIcon( bufferedImage );
//        final JLabel jLabelPlot = new JLabel( imageIcon, JLabel.CENTER );
//        jLabelPlot.setBorder( BorderFactory.createLineBorder( Color.black, 1 ) );
//
//        final JPane jPaneCenter = new JPane( "Center jPane", new Dimension( jPlotLayout.getWidth(), jPlotLayout.getHeight() ) );
//        jPaneCenter.setLayout( new GridBagLayout() );
//        jPaneCenter.add( jLabelPlot,
//                new GridBagConstraints( 0, 0, 1, 1, 0, 0,
//                        GridBagConstraints.CENTER,
//                        GridBagConstraints.CENTER,
//                        new Insets( 0, 0, 0, 0 ), 0, 0 ) );
//
//        return jPaneCenter;

        if( AxeTypeForFixedPlateform.TIME_LINE.name().equals( megapoliPlot.getAxeType() ) || AxeTypeForFixedPlateform.TIME_POINTS.name().equals( megapoliPlot.getAxeType() ) )
        {
            final JPane jPaneGraph = createTimeSeriePlot( megapoliPlot );
            final BufferedImage bufferedImage = new BufferedImage( 600, 600, BufferedImage.TYPE_INT_ARGB );
            final Graphics2D graphics2D = bufferedImage.createGraphics();
//        graphics2D.setBackground( Color.WHITE );
            jPaneGraph.addNotify();
            jPaneGraph.validate();
            jPaneGraph.draw( graphics2D );
            return bufferedImage;
//            final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPaneGraph );
//            return jPanelToImageManager.saveAsImage();
        }
        else
        {
            final JPane jPaneGraph = create2DPlot( megapoliPlot );
            final BufferedImage bufferedImage = new BufferedImage( 600, 600, BufferedImage.TYPE_INT_BGR );
            final Graphics2D graphics2D = bufferedImage.createGraphics();
            jPaneGraph.addNotify();
            jPaneGraph.validate();
            jPaneGraph.draw( graphics2D );
            return bufferedImage;
        }
    }

    /**
     * First layer contains axes, labels and the first set of data
     * Others layers contains only the other sets of data
     *
     * @param megapoliPlot
     * @return
     */
    @NotNull
    public JPane createTimeSeriePlot( @NotNull final MegapoliPlot megapoliPlot )
    {
        final JPane mainPane = new JPane( "zz", new Dimension( 600, 600 ) );
        mainPane.setLayout( new StackedLayout() );
        mainPane.setBorder( BorderFactory.createLineBorder( Color.pink ) );

//        // Graph
////        final JPlotLayoutEther jPlotLayout = new JPlotLayoutEther( false, true, false, "Trajectory data", null, megapoliPlot.isLegendToHide() );
//        final JPlotLayout jPlotLayout = new JPlotLayout( data, "Profile Demo", null, false );
//        jPlotLayout.setTitles( "", "", "" );

        /** ***************************** **/
        /** *********** LINES ************ **/
        /** ***************************** **/
        final double[] dataArray = (double[]) megapoliPlot.getData().getFirstArray();
        final Date[] dateValues = (Date[]) megapoliPlot.getData().getSecondArray();

        final SimpleLine line0 = new SimpleLine( new GeoDateArray( dateValues ), dataArray, "legend" );
        SGTMetaData meta = new SGTMetaData( "", "", false, false );
        line0.setXMetaData( meta );

        meta = new SGTMetaData( megapoliPlot.getParameterName(), megapoliPlot.getParameterUnitCode(), false, false );
        line0.setYMetaData( meta );


        SimpleLine line;
        GeoDate start = null;
        GeoDate stop = null;
        TimeRange tr;
        TestData td;
        try
        {
            start = new GeoDate( "1992-11-01", "yyyy-MM-dd" );
            stop = new GeoDate( "1993-02-20", "yyyy-MM-dd" );
        }
        catch( IllegalTimeValue e )
        {
        }
        tr = new TimeRange( start, stop );
        td = new TestData( TestData.TIME_SERIES, tr, 1.0f,
                TestData.RANDOM, 1.2f, 0.5f, 30.0f );
        line = (SimpleLine) td.getSGTData();


        /** ***************************** **/
        /** *********** AXIS ************ **/
        /** ***************************** **/
        final SoTRange ynRange;
        final SoTRange yRange;
        final SoTRange tnRange;
        final String yLabel;
        yRange = line.getYRange();
//        yRange.add( line2.getYRange() );
        tnRange = line.getXRange();
        final Integer intervalsNumber = 10;
        ynRange = Graph.computeRange( yRange, intervalsNumber );
        yLabel = line.getYMetaData().getName();


        /** ***************************** **/
        /** *********** LAYERS ********** **/
        /** ***************************** **/
        /*
        * xsize, ysize are the width and height in physical units
        * of the Layer graphics region.
        *
        * xstart, xend are the start and end points for the TimeAxis
        * ystart, yend are the start and end points for the Y axis
        */
        final double xsize = 4.0;
        final double xstart = 0.6;
        final double xend = 3.25;
        final double ysize = 4.0;
        final double ystart = 0.6;
        final double yend = 2.50;

        Layer layer2;
        SGLabel label, title, ytitle;
        CartesianGraph graph, graph2;
        LinearTransform xt, yt;
        PlainAxis yleft;
        TimeAxis xbot;
        LineKey lkey;
        GeoDate stime;

        final Layer layer = new Layer( "First Layer", new Dimension2D( xsize, ysize ) );

        /** ***************************** **/
        /** *********** LEGENDS ********* **/
        /** ***************************** **/
        /*
        * create a LineKey
        * the LineKey will be a legend for the two lines created
        * position the key in the upper right corner
        * and add to the first layer
        */
        lkey = new LineKey();
        lkey.setId( "Legend" );
        lkey.setLocationP( new Point2D.Double( xsize - 1, ysize - 1 ) );
        lkey.setVAlign( LineKey.TOP );
        lkey.setHAlign( LineKey.RIGHT );
        layer.addChild( lkey );
        /*
        * add the first layer to the Pane
        */
        mainPane.add( layer );
        /*
        * create first CartesianGraph and transforms
        */
        graph = new CartesianGraph( "First Graph" );
        xt = new LinearTransform( new Range2D( xstart, xend ), tnRange );
        graph.setXTransform( xt );
        yt = new LinearTransform( new Range2D( ystart, yend ), ynRange );
        graph.setYTransform( yt );
        /*
        * Create the time axis, set its range in user units
        * and its origin. Add the axis to the first graph.
        */
        SoTPoint point = new SoTPoint( ynRange.getStart(), tnRange.getStart() );
        xbot = new TimeAxis( "Bottom Axis", TimeAxis.MONTH_YEAR );
        xbot.setRangeU( tnRange );
        xbot.setLocationU( point );
        Font xbfont = new Font( "Helvetica", Font.ITALIC, 14 );
        xbot.setLabelFont( xbfont );
        xbot.setMinorLabelInterval( 1 );
        graph.addXAxis( xbot );
        /*
        * Create the vertical axis, set its range in user units
        * and its origin.  Create the axis title and add the
        * axis to the first graph.
        */
        yleft = new PlainAxis( "Left Axis" );
        yleft.setRangeU( ynRange );
        yleft.setLocationU( point );
        yleft.setLabelFont( xbfont );
        ytitle = new SGLabel( "Y-Axis Title", yLabel,
                new Point2D.Double( 0.0, 0.0 ) );
        Font ytfont = new Font( "Helvetica", Font.PLAIN, 14 );
        ytitle.setFont( ytfont );
        ytitle.setHeightP( 0.2 );
        yleft.setTitle( ytitle );
        graph.addYAxis( yleft );
        /*
        * Add the first graph to the first layer.
        */
        layer.setGraph( graph );
        /*
        * Create a LineAttribute for the display of the first
        * line. Associate the attribute and the line with the
        * first graph.  Add the line to the LineKey.
        */
        LineAttribute attr;

        attr = new LineAttribute( LineAttribute.MARK, 20, Color.red );
        attr.setMarkHeightP( 0.1 );
        graph.setData( line, attr );
        lkey.addLineGraph( (LineCartesianRenderer) graph.getRenderer(),
                new SGLabel( "1st line", "Red Data",
                        new Point2D.Double( 0.0, 0.0 ) ) );
        /*
        * Create the second layer and add it the the Pane.
        * Create the second graph and associate it with the
        * second layer.
        */
//        layer2 = new Layer( "Second Layer", new Dimension2D( xsize, ysize ) );
//        mainPane.add( layer2 );
//        graph2 = new CartesianGraph( "Second Graph", xt, yt );
//        layer2.setGraph( graph2 );
        /*
        * Create a LineAttribute for the display of the second
        * line. Associate the attribute and the line with the
        * second graph.  Add the line to the LineKey.
        */
//        LineAttribute attr2;
//        attr2 = new LineAttribute( LineAttribute.MARK, 2, Color.blue );
//        attr2.setMarkHeightP( 0.1 );
//        graph2.setData( line2, attr2 );
//        lkey.addLineGraph( (LineCartesianRenderer) graph2.getRenderer(),
//                new SGLabel( "2nd line", "Blue Data",
//                        new Point2D.Double( 0.0, 0.0 ) ) );

        return mainPane;
    }

    @NotNull
    public JPane create2DPlot( @NotNull final MegapoliPlot megapoliPlot )
    {
        /*
       * This example uses a pre-created "Layout" for raster time
       * series to simplify the construction of a plot. The
       * JPlotLayout can plot a single grid with
       * a ColorKey, time series with a LineKey, point collection with a
       * PointCollectionKey, and general X-Y plots with a
       * LineKey. JPlotLayout supports zooming, object selection, and
       * object editing.
        */
        SGTData newData;
        TestData td;
        JPlotLayout rpl;
        ContourLevels clevels;
        /*
       * Create a test grid with sinasoidal-ramp data.
        */
        Range2D xr = new Range2D( 190.0f, 250.0f, 1.0f );
        Range2D yr = new Range2D( 0.0f, 45.0f, 1.0f );
        td = new TestData( TestData.XY_GRID, xr, yr,
                TestData.SINE_RAMP, 12.0f, 30.f, 5.0f );
        newData = td.getSGTData();
        /*
       * Create the layout without a Logo image and with the
       * ColorKey on a separate Pane object.
        */
        rpl = new JPlotLayout( true, false, false, "JGridDemo Pane", null, true );
        rpl.setEditClasses( false );
        /*
       * Create a GridAttribute for CONTOUR style.
        */
        Range2D datar = new Range2D( -20.0f, 45.0f, 5.0f );
        clevels = ContourLevels.getDefault( datar );
        GridAttribute gridAttr_ = new GridAttribute( clevels );
        /*
       * Create a ColorMap and change the style to RASTER_CONTOUR.
        */
        ColorMap cmap = createColorMap( datar );
        gridAttr_.setColorMap( cmap );
        gridAttr_.setStyle( GridAttribute.RASTER_CONTOUR );
        /*
       * Add the grid to the layout and give a label for
       * the ColorKey.
        */
        rpl.addData( newData, gridAttr_, "First Data" );
        /*
       * Change the layout's three title lines.
        */
        rpl.setTitles( "Raster Plot Demo",
                "using a JPlotLayout",
                "" );
        /*
       * Resize the graph  and place in the "Center" of the frame.
        */
        rpl.setSize( new Dimension( 600, 400 ) );
        /*
       * Resize the key Pane, both the device size and the physical
       * size. Set the size of the key in physical units and place
       * the key pane at the "South" of the frame.
        */
//    rpl.setKeyLayerSizeP(new Dimension2D(6.0, 1.02));
//    rpl.setKeyBoundsP(new Rectangle2D.Double(0.01, 1.01, 5.98, 1.0));

        return rpl;
    }

    @NotNull
    public JPanel createControlPanel()
    {
        final JPanel controlPanel = new JPanel();
        final ButtonGroup gridtype = new ButtonGroup();
        final JCheckBox stacked = new JCheckBox();
        final JCheckBox grid = new JCheckBox();

        controlPanel.setLayout( new GridBagLayout() );
//    controlPanel.setBackground(new java.awt.Color(200,200,200));
        controlPanel.setBounds( 0, 679, 426, 33 );
        controlPanel.setBorder( new EtchedBorder() );
        gridtype.add( stacked );
        stacked.setSelected( true );
        stacked.setText( "Overlayed" );

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.weightx = 0.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets( 5, 15, 5, 5 );
        gbc.ipadx = 0;
        gbc.ipady = 0;
        controlPanel.add( stacked, gbc );
        stacked.setBounds( 15, 5, 84, 23 );
        gridtype.add( grid );
        grid.setText( "Grid" );

        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.weightx = 0.5;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.insets = new Insets( 5, 5, 5, 0 );
        gbc.ipadx = 0;
        gbc.ipady = 0;
        controlPanel.add( grid, gbc );
        grid.setBounds( 109, 5, 53, 23 );
        final JButton showTree = new JButton();
        showTree.setText( "Show Class Tree" );

        gbc.gridx = 2;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.insets = new Insets( 5, 5, 5, 15 );
        gbc.ipadx = 0;
        gbc.ipady = 0;
        controlPanel.add( showTree, gbc );
        showTree.setBackground( Color.yellow.brighter() );
        showTree.setBounds( 302, 5, 109, 23 );

        final SymItem lSymItem = new SymItem( stacked, grid );
        stacked.addItemListener( lSymItem );
        grid.addItemListener( lSymItem );
//        SymAction lSymAction = new SymAction();
//        showTree.addActionListener( lSymAction );

        return controlPanel;
    }

    /**
     * Create the center JPane with the graph
     *
     * @param megapoliPlot
     * @param maxWidth
     * @param maxHeight
     * @return
     */
    @NotNull
    public JPane createCenterPaneOld( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer maxWidth, @NotNull final Integer maxHeight, @Nullable final Locale locale )
    {
        final Integer plotWidth = Math.min( maxWidth - MARGIN_LEFT_RIGHT, maxWidth );
        final Integer plotHeight = Math.min( maxHeight - MARGIN_LEFT_RIGHT, maxHeight );

        final JPlotLayout jPlotLayout;
        final BufferedImage bufferedImage;
        if( megapoliPlot.isTimeSerie() )
        {
            jPlotLayout = createJPlotLayoutForTimeSerie( megapoliPlot, plotWidth, plotHeight );

            // TODO : remove this change to bufferedImage to insert directly the jPlotLayout !!!!
            bufferedImage = new BufferedImage( jPlotLayout.getWidth(), jPlotLayout.getHeight(), BufferedImage.TYPE_INT_RGB );
            final Graphics2D graphics2D = bufferedImage.createGraphics();
            jPlotLayout.draw( graphics2D );
        }
        else
        {
            jPlotLayout = createJPlotLayoutFor2D( megapoliPlot, plotWidth, plotHeight, locale );

            final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPlotLayout );
            bufferedImage = jPanelToImageManager.saveAsImage();
        }

        final ImageIcon imageIcon = new ImageIcon( bufferedImage );
        final JLabel jLabelPlot = new JLabel( imageIcon, JLabel.CENTER );
        jLabelPlot.setBorder( BorderFactory.createLineBorder( Color.black, 1 ) );

        final JPane jPaneCenter = new JPane( "Center jPane", new Dimension( jPlotLayout.getWidth(), jPlotLayout.getHeight() ) );
        jPaneCenter.setLayout( new GridBagLayout() );
        jPaneCenter.add( jLabelPlot,
                new GridBagConstraints( 0, 0, 1, 1, 0, 0,
                        GridBagConstraints.CENTER,
                        GridBagConstraints.CENTER,
                        new Insets( 0, 0, 0, 0 ), 0, 0 ) );

        return jPaneCenter;
    }

    /**
     * Change the Pane layout to StackedLayout.
     *
     * @param event
     */

    void stacked_itemStateChanged( java.awt.event.ItemEvent event )
    {
        /*
        * Get the component list for mainPane_ and change
        * the layout to StackedLayout.
        */
//        Component[] comps = mainPane_.getComponents();
//        mainPane_.setBatch( true );
//        mainPane_.setLayout( new StackedLayout() );
//        /*
//        * Remove any axes that have been associated with
//        * the second graph.  With the layers overlayed it
//        * is not necessary to have duplicate axes.
//        */
//        Graph gr2 = ( (Layer) comps[1] ).getGraph();
//        ( (CartesianGraph) gr2 ).removeAllXAxes();
//        ( (CartesianGraph) gr2 ).removeAllYAxes();
//        /*
//        * Tell the Applet that the mainPane_ needs to
//        * be layed out and re-draw the mainPane_.
//        */
//        if( isApplet_ )
//        {
//            validate();
//        }
//        else
//        {
//            frame.validate();
//        }
//        mainPane_.setBatch( false );
//        if( tree_ != null )
//        {
//            if( tree_.isVisible() )
//            {
//                tree_.setJPane( mainPane_ );
//                tree_.expandTree();
//            }
//        }
    }

    /**
     * Change the Pane layout to GridLayout.
     *
     * @param event
     */

    void grid_itemStateChanged( java.awt.event.ItemEvent event )
    {
        /*
        * Get the component list for mainPane_ and change
        * the layout to GridLayout.
        */
//        Component[] comps = mainPane_.getComponents();
//        mainPane_.setBatch( true );
//        mainPane_.setLayout( new GridLayout( 2, 0 ) );
//        /*
//        * Get the first and second graphs from the first
//        * and second layers, respectively.
//        */
//        Graph gr = ( (Layer) comps[0] ).getGraph();
//        Graph gr2 = ( (Layer) comps[1] ).getGraph();
//        /*
//        * Create copies of all X-Axes associated with the first
//        * graph for the second graph. If the axes are not copied then
//        * the second graph will have the second line plotted, but without
//        * any axes.
//        */
//        for( Enumeration xa = ( (CartesianGraph) gr ).xAxisElements(); xa.hasMoreElements(); )
//        {
//            ( (CartesianGraph) gr2 ).addXAxis( ( (Axis) xa.nextElement() ).copy() );
//        }
//        /*
//        * Create copies of all Y-Axes associated with the first
//        * graph for the second graph.
//        */
//        for( Enumeration ya = ( (CartesianGraph) gr ).yAxisElements(); ya.hasMoreElements(); )
//        {
//            ( (CartesianGraph) gr2 ).addYAxis( ( (Axis) ya.nextElement() ).copy() );
//        }
//        /*
//        * Tell the Applet that the mainPane_ needs to
//        * be layed out and re-draw the mainPane_.
//        */
//        if( isApplet_ )
//        {
//            validate();
//        }
//        else
//        {
//            frame.validate();
//        }
//        //    mainPane_.draw();
//        mainPane_.setBatch( false );
//        if( tree_ != null )
//        {
//            if( tree_.isVisible() )
//            {
//                tree_.setJPane( mainPane_ );
//                tree_.expandTree();
//            }
//        }
    }

    void showTree_ActionPerformed( java.awt.event.ActionEvent event )
    {
//        /*
//        * Create the ClassTree dialog to display the classes used
//        * in the mainPane_ and allow editing.
//        */
//        if( tree_ == null )
//        {
//            tree_ = new JClassTree( "Classes for LayoutDemo" );
//        }
//        tree_.setJPane( mainPane_ );
//        tree_.show();
    }

    @NotNull
    private JPlotLayout createJPlotLayoutForTimeSerie( final MegapoliPlot megapoliPlot, final Integer plotWidth, final Integer plotHeight )
    {
        final double[] dataArray = (double[]) megapoliPlot.getData().getFirstArray();
        final Date[] dateValues = (Date[]) megapoliPlot.getData().getSecondArray();

        final SimpleLine data = new SimpleLine( new GeoDateArray( dateValues ), dataArray, "legend" );
        SGTMetaData meta = new SGTMetaData( null, null, false, false );
        data.setXMetaData( meta );

        meta = new SGTMetaData( megapoliPlot.getParameterName(), megapoliPlot.getParameterUnitCode(), false, false );
        data.setYMetaData( meta );

        // Graph
//        final JPlotLayoutEther jPlotLayout = new JPlotLayoutEther( false, true, false, "Trajectory data", null, megapoliPlot.isLegendToHide() );
        final JPlotLayout jPlotLayout = new JPlotLayout( data, "Profile Demo", null, false );
        jPlotLayout.setTitles( "", "", "" );

        // Replace legend
//        final String legend = megapoliPlot.getData().getTitle();
//        final Dimension2D layerSizeP = jPlotLayout.getLayerSizeP();
//        final double factorForPlotWidthInPhysicalUnits = layerSizeP.getWidth() / Double.valueOf( plotWidth );
//        final double legendSizeInPhysicalUnits = legend.length() * factorForPlotWidthInPhysicalUnits * 7;
//        jPlotLayout.setKeyLocationP( new Point2D.Double( layerSizeP.getWidth() - legendSizeInPhysicalUnits, layerSizeP.getHeight() - 0.1 ) );

        // Add data and legend
        jPlotLayout.addData( data, "" );
        jPlotLayout.setSize( plotWidth, plotHeight );
        jPlotLayout.setVisible( true );

        jPlotLayout.addNotify();
        jPlotLayout.validate();
        jPlotLayout.setOpaque( true );
        return jPlotLayout;
    }

    @NotNull
    private JPlotLayout createJPlotLayoutFor2D( final MegapoliPlot megapoliPlot, final Integer plotWidth, final Integer plotHeight, @Nullable final Locale locale )
    {
        SGTData newData;
        TestData td;
        JPlotLayout rpl;
        ContourLevels clevels;
        /*
       * Create a test grid with sinasoidal-ramp data.
        */
        Range2D xr = new Range2D( 190.0f, 250.0f, 1.0f );
        Range2D yr = new Range2D( 0.0f, 45.0f, 1.0f );
        td = new TestData( TestData.XY_GRID, xr, yr,
                TestData.SINE_RAMP, 12.0f, 30.f, 5.0f );
        newData = td.getSGTData();
        /*
       * Create the layout without a Logo image and with the
       * ColorKey on a separate Pane object.
        */
        rpl = new JPlotLayout( true, false, false, "JGridDemo Pane", null, true );
        rpl.setEditClasses( false );
        /*
       * Create a GridAttribute for CONTOUR style.
        */
        Range2D datar = new Range2D( -20.0f, 45.0f, 5.0f );
        clevels = ContourLevels.getDefault( datar );
        final GridAttribute gridAttr_ = new GridAttribute( clevels );
        /*
       * Create a ColorMap and change the style to RASTER_CONTOUR.
        */
        ColorMap cmap = createColorMap( datar );
        gridAttr_.setColorMap( cmap );
        gridAttr_.setStyle( GridAttribute.RASTER_CONTOUR );
        /*
       * Add the grid to the layout and give a label for
       * the ColorKey.
        */
        rpl.addData( newData, gridAttr_, "First Data" );
        /*
       * Change the layout's three title lines.
        */
        rpl.setTitles( "Raster Plot Demo", "using a JPlotLayout", "" );
        /*
       * Resize the graph  and place in the "Center" of the frame.
        */
        rpl.setSize( new Dimension( 600, 400 ) );
        /*
       * Resize the key Pane, both the device size and the physical
       * size. Set the size of the key in physical units and place
       * the key pane at the "South" of the frame.
        */
//    rpl.setKeyLayerSizeP(new Dimension2D(6.0, 1.02));
//    rpl.setKeyBoundsP(new Rectangle2D.Double(0.01, 1.01, 5.98, 1.0));

        return rpl;
    }

    @NotNull
    private JPlotLayout createJPlotLayoutFor2DOrig( final MegapoliPlot megapoliPlot, final Integer plotWidth, final Integer plotHeight, @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );

        final double[] parameterArray = (double[]) megapoliPlot.getData().getFirstArray();
        final double[] latitudeArray = (double[]) megapoliPlot.getData().getSecondArray();
        final double[] longitudeValues = (double[]) megapoliPlot.getData().getThirdArray();

//        final SimpleLine data = new SimpleLine( longitudeValues, latitudeArray, "legend" );
        final SimpleGrid data = new SimpleGrid( parameterArray, longitudeValues, latitudeArray, "Test Series" );
        SGTMetaData meta = new SGTMetaData( bundle.getString( "plot.longitude" ), bundle.getString( "plot.degres" ), false, true );
        // TODO : vérifier modulo = 360
//        meta.setModuloValue( 360 );
        data.setXMetaData( meta );

        meta = new SGTMetaData( bundle.getString( "plot.latitude" ), bundle.getString( "plot.degres" ), false, true );
        // TODO : vérifier modulo = 360
//        meta.setModuloValue( 360 );
        data.setYMetaData( meta );

        // Graph
        final JPlotLayout jPlotLayout = new JPlotLayout( true, false, false, "JGridDemo Pane", null, true );
        jPlotLayout.setEditClasses( false );
        /*
       * Create a GridAttribute for CONTOUR style.
        */
        final Range2D datar = new Range2D( -20.0f, 45.0f, 5.0f );
        final ContourLevels clevels = ContourLevels.getDefault( datar );
        final GridAttribute gridAttr = new GridAttribute( clevels );
        /*
       * Create a ColorMap and change the style to RASTER_CONTOUR.
        */
        final ColorMap cmap = createColorMap( datar );
        gridAttr.setColorMap( cmap );
        gridAttr.setStyle( GridAttribute.RASTER_CONTOUR );
        /*
       * Add the grid to the layout and give a label for
       * the ColorKey.
        */
        jPlotLayout.addData( data, gridAttr, "First Data" );

//        final JPlotLayoutEther jPlotLayout = new JPlotLayoutEther( false, megapoliPlot.isXTime(), false, "Trajectory data", null, megapoliPlot.isLegendToHide() );
//        jPlotLayout.setTitles( "", "", "" );

        // Replace legend
//        final String legend = megapoliPlot.getData().getTitle();
//        final Dimension2D layerSizeP = jPlotLayout.getLayerSizeP();
//        final double factorForPlotWidthInPhysicalUnits = layerSizeP.getWidth() / Double.valueOf( plotWidth );
//        final double legendSizeInPhysicalUnits = legend.length() * factorForPlotWidthInPhysicalUnits * 7;
//        jPlotLayout.setKeyLocationP( new Point2D.Double( layerSizeP.getWidth() - legendSizeInPhysicalUnits, layerSizeP.getHeight() - 0.1 ) );

        // Add data and legend
//        jPlotLayout.addData( megapoliPlot.getData(), legend );
        jPlotLayout.setSize( plotWidth, plotHeight );
        jPlotLayout.setVisible( true );

        jPlotLayout.addNotify();
        jPlotLayout.validate();
        jPlotLayout.setOpaque( true );
        return jPlotLayout;
    }

    @NotNull
    private String formatTitle( @Nullable final String title )
    {
        if( null == title )
            return "";
        else
        {
            final String formatTitle = title.replaceAll( "\\n", "<br>" );
            return "<html>" + formatTitle + "</html>";
        }
    }

    @NotNull
    private ColorMap createColorMap( final Range2D datar )
    {
        final int[] red =
                {0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 7, 23, 39, 55, 71, 87, 103,
                        119, 135, 151, 167, 183, 199, 215, 231,
                        247, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 246, 228, 211, 193, 175, 158, 140};
        final int[] green =
                {0, 0, 0, 0, 0, 0, 0, 0,
                        0, 11, 27, 43, 59, 75, 91, 107,
                        123, 139, 155, 171, 187, 203, 219, 235,
                        251, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 247, 231, 215, 199, 183, 167, 151,
                        135, 119, 103, 87, 71, 55, 39, 23,
                        7, 0, 0, 0, 0, 0, 0, 0};
        final int[] blue =
                {0, 143, 159, 175, 191, 207, 223, 239,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 247, 231, 215, 199, 183, 167, 151,
                        135, 119, 103, 87, 71, 55, 39, 23,
                        7, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0};

        final IndexedColorMap cmap = new IndexedColorMap( red, green, blue );
        cmap.setTransform( new LinearTransform( 0.0, (double) red.length, datar.start, datar.end ) );
        return cmap;
    }

    private static final Log LOGGER = LogFactory.getLog( EtherPlotServiceImpl.class );

    // Dimensions of the jPanes
    private static final int MAX_WIDTH = 800;
    private static final int MAX_HEIGHT = 700;
    private static final int MARGIN_LEFT_RIGHT = 50;

    private static final int TITLE_FONT_SIZE = 20;
    private static final int FONT_SIZE = 12;
    private static final int ERROR_FONT_SIZE = 10;
}
